@@ -9,23 +9,7 @@
  * Copyright (c) 2007, Mateusz Loskot
  * Copyright (c) 2010-2013, Even Rouault <even dot rouault at spatialys.com>
  *
- * Permission is hereby granted, free of charge, to any person obtaining a
- * copy of this software and associated documentation files (the "Software"),
- * to deal in the Software without restriction, including without limitation
- * the rights to use, copy, modify, merge, publish, distribute, sublicense,
- * and/or sell copies of the Software, and to permit persons to whom the
- * Software is furnished to do so, subject to the following conditions:
- *
- * The above copyright notice and this permission notice shall be included
- * in all copies or substantial portions of the Software.
- *
- * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS
- * OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
- * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL
- * THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
- * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
- * FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
- * DEALINGS IN THE SOFTWARE.
+ * SPDX-License-Identifier: MIT
  ****************************************************************************/
 #ifndef OGR_GEOJSONREADER_H_INCLUDED
 #define OGR_GEOJSONREADER_H_INCLUDED
@@ -61,33 +45,6 @@ class OGRFeature;
 class OGRGeoJSONLayer;
 class OGRSpatialReference;
 
-/************************************************************************/
-/*                           GeoJSONObject                              */
-/************************************************************************/
-
-struct GeoJSONObject
-{
-    enum Type
-    {
-        eUnknown = wkbUnknown,  // non-GeoJSON properties
-        ePoint = wkbPoint,
-        eLineString = wkbLineString,
-        ePolygon = wkbPolygon,
-        eMultiPoint = wkbMultiPoint,
-        eMultiLineString = wkbMultiLineString,
-        eMultiPolygon = wkbMultiPolygon,
-        eGeometryCollection = wkbGeometryCollection,
-        eFeature,
-        eFeatureCollection
-    };
-
-    enum CoordinateDimension
-    {
-        eMinCoordinateDimension = 2,
-        eMaxCoordinateDimension = 3
-    };
-};
-
 /************************************************************************/
 /*                        OGRGeoJSONBaseReader                          */
 /************************************************************************/
@@ -258,21 +215,6 @@ void OGRGeoJSONReaderAddOrUpdateField(
 /*                 GeoJSON Parsing Utilities                            */
 /************************************************************************/
 
-lh_entry *OGRGeoJSONFindMemberEntryByName(json_object *poObj,
-                                          const char *pszName);
-json_object *OGRGeoJSONFindMemberByName(json_object *poObj,
-                                        const char *pszName);
-GeoJSONObject::Type OGRGeoJSONGetType(json_object *poObj);
-
-json_object CPL_DLL *json_ex_get_object_by_path(json_object *poObj,
-                                                const char *pszPath);
-
-json_object CPL_DLL *CPL_json_object_object_get(struct json_object *obj,
-                                                const char *key);
-
-bool CPL_DLL OGRJSonParse(const char *pszText, json_object **ppoObj,
-                          bool bVerboseError = true);
-
 bool OGRGeoJSONUpdateLayerGeomType(bool &bFirstGeom,
                                    OGRwkbGeometryType eGeomType,
                                    OGRwkbGeometryType &eLayerGeomType);
@@ -280,28 +222,6 @@ bool OGRGeoJSONUpdateLayerGeomType(bool &bFirstGeom,
 // Get the 3D extent from the geometry coordinates of a feature
 bool OGRGeoJSONGetExtent3D(json_object *poObj, OGREnvelope3D *poEnvelope);
 
-/************************************************************************/
-/*                 GeoJSON Geometry Translators                         */
-/************************************************************************/
-
-OGRwkbGeometryType OGRGeoJSONGetOGRGeometryType(json_object *poObj);
-
-bool OGRGeoJSONReadRawPoint(json_object *poObj, OGRPoint &point);
-OGRGeometry CPL_DLL *
-OGRGeoJSONReadGeometry(json_object *poObj,
-                       OGRSpatialReference *poParentSRS = nullptr);
-OGRPoint *OGRGeoJSONReadPoint(json_object *poObj);
-OGRMultiPoint *OGRGeoJSONReadMultiPoint(json_object *poObj);
-OGRLineString *OGRGeoJSONReadLineString(json_object *poObj, bool bRaw = false);
-OGRMultiLineString *OGRGeoJSONReadMultiLineString(json_object *poObj);
-OGRLinearRing *OGRGeoJSONReadLinearRing(json_object *poObj);
-OGRPolygon *OGRGeoJSONReadPolygon(json_object *poObj, bool bRaw = false);
-OGRMultiPolygon *OGRGeoJSONReadMultiPolygon(json_object *poObj);
-OGRGeometryCollection *
-OGRGeoJSONReadGeometryCollection(json_object *poObj,
-                                 OGRSpatialReference *poSRS = nullptr);
-OGRSpatialReference *OGRGeoJSONReadSpatialReference(json_object *poObj);
-
 /************************************************************************/
 /*                          OGRESRIJSONReader                           */
 /************************************************************************/
@@ -341,14 +261,6 @@ class OGRESRIJSONReader
     OGRGeoJSONLayer *ReadFeatureCollection(json_object *poObj);
 };
 
-OGRGeometry *OGRESRIJSONReadGeometry(json_object *poObj);
-OGRSpatialReference *OGRESRIJSONReadSpatialReference(json_object *poObj);
-OGRwkbGeometryType OGRESRIJSONGetGeometryType(json_object *poObj);
-OGRPoint *OGRESRIJSONReadPoint(json_object *poObj);
-OGRGeometry *OGRESRIJSONReadLineString(json_object *poObj);
-OGRGeometry *OGRESRIJSONReadPolygon(json_object *poObj);
-OGRMultiPoint *OGRESRIJSONReadMultiPoint(json_object *poObj);
-
 /************************************************************************/
 /*                          OGRTopoJSONReader                           */
 /************************************************************************/
