@@ -118,11 +118,14 @@ int swqlex(YYSTYPE *ppNode, swq_parse_context *context)
 
         while (*pszInput != '\0')
         {
-            if (chQuote == '"' && *pszInput == '\\' && pszInput[1] == '"')
-                pszInput++;
-            else if (chQuote == '\'' && *pszInput == '\\' &&
-                     pszInput[1] == '\'')
+            // Not totally sure we need to preserve this way of escaping for
+            // strings between double-quotes
+            if (chQuote == '"' && *pszInput == '\\')
+            {
                 pszInput++;
+                if (*pszInput == '\0')
+                    break;
+            }
             else if (chQuote == '\'' && *pszInput == '\'' &&
                      pszInput[1] == '\'')
                 pszInput++;
@@ -142,7 +145,7 @@ int swqlex(YYSTYPE *ppNode, swq_parse_context *context)
             CPLError(CE_Failure, CPLE_AppDefined,
                      "Did not find end-of-string character");
             CPLFree(token);
-            return 0;
+            return YYerror;
         }
 
         *ppNode = new swq_expr_node(token);
@@ -216,7 +219,7 @@ int swqlex(YYSTYPE *ppNode, swq_parse_context *context)
     /* -------------------------------------------------------------------- */
     /*      Handle alpha-numerics.                                          */
     /* -------------------------------------------------------------------- */
-    else if (isalnum(*pszInput))
+    else if (isalnum(static_cast<unsigned char>(*pszInput)))
     {
         int nReturn = SWQT_IDENTIFIER;
         CPLString osToken;
@@ -225,8 +228,8 @@ int swqlex(YYSTYPE *ppNode, swq_parse_context *context)
         osToken += *pszInput;
 
         // collect text characters
-        while (isalnum(*pszNext) || *pszNext == '_' ||
-               static_cast<unsigned char>(*pszNext) > 127)
+        while (isalnum(static_cast<unsigned char>(*pszNext)) ||
+               *pszNext == '_' || static_cast<unsigned char>(*pszNext) > 127)
             osToken += *(pszNext++);
 
         context->pszNext = pszNext;
@@ -239,6 +242,10 @@ int swqlex(YYSTYPE *ppNode, swq_parse_context *context)
             nReturn = SWQT_ILIKE;
         else if (EQUAL(osToken, "ESCAPE"))
             nReturn = SWQT_ESCAPE;
+        else if (EQUAL(osToken, "EXCEPT"))
+            nReturn = SWQT_EXCEPT;
+        else if (EQUAL(osToken, "EXCLUDE"))
+            nReturn = SWQT_EXCLUDE;
         else if (EQUAL(osToken, "NULL"))
             nReturn = SWQT_NULL;
         else if (EQUAL(osToken, "IS"))
@@ -285,6 +292,11 @@ int swqlex(YYSTYPE *ppNode, swq_parse_context *context)
             nReturn = SWQT_LIMIT;
         else if (EQUAL(osToken, "OFFSET"))
             nReturn = SWQT_OFFSET;
+        else if (EQUAL(osToken, "HIDDEN"))
+        {
+            *ppNode = new swq_expr_node(osToken);
+            nReturn = SWQT_HIDDEN;
+        }
 
         // Unhandled by OGR SQL.
         else if (EQUAL(osToken, "OUTER") || EQUAL(osToken, "INNER"))
@@ -314,20 +326,20 @@ int swqlex(YYSTYPE *ppNode, swq_parse_context *context)
 /************************************************************************/
 
 const char *swq_select_summarize(swq_select *select_info, int dest_column,
-                                 const char *value)
+                                 const char *pszValue, const double *pdfValue)
 
 {
-    swq_col_def *def = select_info->column_defs + dest_column;
-
     /* -------------------------------------------------------------------- */
     /*      Do various checking.                                            */
     /* -------------------------------------------------------------------- */
     if (select_info->query_mode == SWQM_RECORDSET)
         return "swq_select_summarize() called on non-summary query.";
 
-    if (dest_column < 0 || dest_column >= select_info->result_columns)
+    if (dest_column < 0 ||
+        dest_column >= static_cast<int>(select_info->column_defs.size()))
         return "dest_column out of range in swq_select_summarize().";
 
+    const swq_col_def *def = &select_info->column_defs[dest_column];
     if (def->col_func == SWQCF_NONE && !def->distinct_flag)
         return nullptr;
 
@@ -348,8 +360,8 @@ const char *swq_select_summarize(swq_select *select_info, int dest_column,
     /* -------------------------------------------------------------------- */
     if (select_info->column_summary.empty())
     {
-        select_info->column_summary.resize(select_info->result_columns);
-        for (int i = 0; i < select_info->result_columns; i++)
+        select_info->column_summary.resize(select_info->column_defs.size());
+        for (std::size_t i = 0; i < select_info->column_defs.size(); i++)
         {
             if (def->distinct_flag)
             {
@@ -357,7 +369,7 @@ const char *swq_select_summarize(swq_select *select_info, int dest_column,
                 if (select_info->order_specs > 0)
                 {
                     CPLAssert(select_info->order_specs == 1);
-                    CPLAssert(select_info->result_columns == 1);
+                    CPLAssert(select_info->column_defs.size() == 1);
                     oComparator.bSortAsc =
                         CPL_TO_BOOL(select_info->order_defs[0].ascending_flag);
                 }
@@ -394,18 +406,17 @@ const char *swq_select_summarize(swq_select *select_info, int dest_column,
 
     if (def->distinct_flag)
     {
-        if (value == nullptr)
-            value = SZ_OGR_NULL;
+        if (pszValue == nullptr)
+            pszValue = SZ_OGR_NULL;
         try
         {
-            if (summary.oSetDistinctValues.find(value) ==
-                summary.oSetDistinctValues.end())
+            if (!cpl::contains(summary.oSetDistinctValues, pszValue))
             {
-                summary.oSetDistinctValues.insert(value);
+                summary.oSetDistinctValues.insert(pszValue);
                 if (select_info->order_specs == 0)
                 {
                     // If not sorted, keep values in their original order
-                    summary.oVectorDistinctValues.emplace_back(value);
+                    summary.oVectorDistinctValues.emplace_back(pszValue);
                 }
                 summary.count++;
             }
@@ -425,57 +436,78 @@ const char *swq_select_summarize(swq_select *select_info, int dest_column,
     switch (def->col_func)
     {
         case SWQCF_MIN:
-            if (value != nullptr && value[0] != '\0')
+            if (pdfValue)
             {
-                if (def->field_type == SWQ_DATE ||
-                    def->field_type == SWQ_TIME ||
-                    def->field_type == SWQ_TIMESTAMP)
+                if (*pdfValue < summary.min)
+                    summary.min = *pdfValue;
+                summary.count++;
+            }
+            else if (pszValue && pszValue[0] != '\0')
+            {
+                if (summary.count == 0 || strcmp(pszValue, summary.osMin) < 0)
                 {
-                    if (strcmp(value, summary.osMin) < 0)
-                    {
-                        summary.osMin = value;
-                    }
-                }
-                else
-                {
-                    double df_val = CPLAtof(value);
-                    if (df_val < summary.min)
-                        summary.min = df_val;
+                    summary.osMin = pszValue;
                 }
                 summary.count++;
             }
             break;
         case SWQCF_MAX:
-            if (value != nullptr && value[0] != '\0')
+            if (pdfValue)
             {
-                if (def->field_type == SWQ_DATE ||
-                    def->field_type == SWQ_TIME ||
-                    def->field_type == SWQ_TIMESTAMP)
+                if (*pdfValue > summary.max)
+                    summary.max = *pdfValue;
+                summary.count++;
+            }
+            else if (pszValue && pszValue[0] != '\0')
+            {
+                if (summary.count == 0 || strcmp(pszValue, summary.osMax) > 0)
                 {
-                    if (strcmp(value, summary.osMax) > 0)
-                    {
-                        summary.osMax = value;
-                    }
-                }
-                else
-                {
-                    double df_val = CPLAtof(value);
-                    if (df_val > summary.max)
-                        summary.max = df_val;
+                    summary.osMax = pszValue;
                 }
                 summary.count++;
             }
             break;
         case SWQCF_AVG:
         case SWQCF_SUM:
-            if (value != nullptr && value[0] != '\0')
+            if (pdfValue)
+            {
+                summary.count++;
+
+                // Cf KahanBabushkaNeumaierSum of
+                // https://en.wikipedia.org/wiki/Kahan_summation_algorithm#Further_enhancements
+                // We set a number of temporary variables as volatile, to
+                // prevent potential undesired compiler optimizations.
+
+                const double dfNewVal = *pdfValue;
+                const volatile double new_sum_acc = summary.sum_acc + dfNewVal;
+                if (summary.sum_only_finite_terms && std::isfinite(dfNewVal))
+                {
+                    if (std::fabs(summary.sum_acc) >= std::fabs(dfNewVal))
+                    {
+                        const volatile double diff =
+                            (summary.sum_acc - new_sum_acc);
+                        summary.sum_correction += (diff + dfNewVal);
+                    }
+                    else
+                    {
+                        const volatile double diff = (dfNewVal - new_sum_acc);
+                        summary.sum_correction += (diff + summary.sum_acc);
+                    }
+                }
+                else
+                {
+                    summary.sum_only_finite_terms = false;
+                }
+                summary.sum_acc = new_sum_acc;
+            }
+            else if (pszValue && pszValue[0] != '\0')
             {
                 if (def->field_type == SWQ_DATE ||
                     def->field_type == SWQ_TIME ||
                     def->field_type == SWQ_TIMESTAMP)
                 {
                     OGRField sField;
-                    if (OGRParseDate(value, &sField, 0))
+                    if (OGRParseDate(pszValue, &sField, 0))
                     {
                         struct tm brokendowntime;
                         brokendowntime.tm_year = sField.Date.Year - 1900;
@@ -486,36 +518,84 @@ const char *swq_select_summarize(swq_select *select_info, int dest_column,
                         brokendowntime.tm_sec =
                             static_cast<int>(sField.Date.Second);
                         summary.count++;
-                        summary.sum += CPLYMDHMSToUnixTime(&brokendowntime);
-                        summary.sum +=
+                        summary.sum_acc += CPLYMDHMSToUnixTime(&brokendowntime);
+                        summary.sum_acc +=
                             fmod(static_cast<double>(sField.Date.Second), 1.0);
                     }
                 }
                 else
                 {
-                    summary.count++;
-                    summary.sum += CPLAtof(value);
+                    return "swq_select_summarize() - AVG()/SUM() called on "
+                           "unexpected field type";
                 }
             }
             break;
 
         case SWQCF_COUNT:
-            if (value != nullptr)
+            if (pdfValue || pszValue)
                 summary.count++;
             break;
 
+        case SWQCF_STDDEV_POP:
+        case SWQCF_STDDEV_SAMP:
+        {
+            const auto UpdateVariance = [&summary](double dfValue)
+            {
+                // Welford's online algorithm for variance:
+                // https://en.wikipedia.org/wiki/Algorithms_for_calculating_variance#Welford's_online_algorithm
+                summary.count++;
+                const double dfDelta = dfValue - summary.mean_for_variance;
+                summary.mean_for_variance += dfDelta / summary.count;
+                const double dfDelta2 = dfValue - summary.mean_for_variance;
+                summary.sq_dist_from_mean_acc += dfDelta * dfDelta2;
+            };
+
+            if (pdfValue)
+            {
+                UpdateVariance(*pdfValue);
+            }
+            else if (pszValue && pszValue[0] != '\0')
+            {
+                if (def->field_type == SWQ_DATE ||
+                    def->field_type == SWQ_TIME ||
+                    def->field_type == SWQ_TIMESTAMP)
+                {
+                    OGRField sField;
+                    if (OGRParseDate(pszValue, &sField, 0))
+                    {
+                        struct tm brokendowntime;
+                        brokendowntime.tm_year = sField.Date.Year - 1900;
+                        brokendowntime.tm_mon = sField.Date.Month - 1;
+                        brokendowntime.tm_mday = sField.Date.Day;
+                        brokendowntime.tm_hour = sField.Date.Hour;
+                        brokendowntime.tm_min = sField.Date.Minute;
+                        brokendowntime.tm_sec =
+                            static_cast<int>(sField.Date.Second);
+
+                        UpdateVariance(static_cast<double>(
+                            CPLYMDHMSToUnixTime(&brokendowntime)));
+                    }
+                }
+                else
+                {
+                    return "swq_select_summarize() - STDDEV() called on "
+                           "unexpected field type";
+                }
+            }
+
+            break;
+        }
+
         case SWQCF_NONE:
             break;
 
         case SWQCF_CUSTOM:
             return "swq_select_summarize() called on custom field function.";
-
-        default:
-            return "swq_select_summarize() - unexpected col_func";
     }
 
     return nullptr;
 }
+
 /************************************************************************/
 /*                      sort comparison functions.                      */
 /************************************************************************/
